# Cloudinary Analysis Python SDK

<!-- Start Summary [summary] -->
## Summary

Analyze API (Beta): Use the Analyze API to analyze any external asset and return details based on the type of analysis requested.

Currently supports the following analysis options:
  * [AI Vision - Tagging](https://cloudinary.com/documentation/cloudinary_ai_vision_addon#tagging_mode)
  * [AI Vision - Moderation](https://cloudinary.com/documentation/cloudinary_ai_vision_addon#moderation_mode)
  * [AI Vision - General](https://cloudinary.com/documentation/cloudinary_ai_vision_addon#general_mode)
  * [Captioning](https://cloudinary.com/documentation/cloudinary_ai_content_analysis_addon#ai_based_image_captioning)
  * [Cld Fashion](https://cloudinary.com/documentation/cloudinary_ai_content_analysis_addon#supported_content_aware_detection_models)
  * [Cld Text](https://cloudinary.com/documentation/cloudinary_ai_content_analysis_addon#supported_content_aware_detection_models)
  * [Coco](https://cloudinary.com/documentation/cloudinary_ai_content_analysis_addon#supported_content_aware_detection_models)
  * [Google Tagging](https://cloudinary.com/documentation/google_auto_tagging_addon)
  * [Human Anatomy](https://cloudinary.com/documentation/cloudinary_ai_content_analysis_addon#supported_content_aware_detection_models)
  * [Image Quality Analysis](https://cloudinary.com/documentation/cloudinary_ai_content_analysis_addon#image_quality_analysis)
  * [Lvis](https://cloudinary.com/documentation/cloudinary_ai_content_analysis_addon#supported_content_aware_detection_models)
  * [Shop Classifier](https://cloudinary.com/documentation/cloudinary_ai_content_analysis_addon#supported_content_aware_detection_models)
  * [Unidet](https://cloudinary.com/documentation/cloudinary_ai_content_analysis_addon#supported_content_aware_detection_models)
  * [Watermark Detection](https://cloudinary.com/documentation/cloudinary_ai_content_analysis_addon#watermark_detection)

  **Notes**: 
  * The Analyze API is currently in development and is available as a Public Beta, which means we value your feedback, so please feel free to [share any thoughts with us](https://support.cloudinary.com/hc/en-us/requests/new).
  * The analysis options require an active subscription to the relevant add-on. Learn more about [registering for add-ons](https://cloudinary.com/documentation/cloudinary_add_ons#registering_for_add_ons).

  The API supports both Basic Authentication using your Cloudinary API Key and API Secret (which can be found on the Dashboard page of your [Cloudinary Console](https://console.cloudinary.com/pm)) or OAuth2 ([Contact support](https://support.cloudinary.com/hc/en-us/requests/new) for more information regarding OAuth).
<!-- End Summary [summary] -->

<!-- Start Table of Contents [toc] -->
## Table of Contents
<!-- $toc-max-depth=2 -->
* [Cloudinary Analysis Python SDK](https://github.com/cloudinary/analysis-python/blob/master/#cloudinary-analysis-python-sdk)
  * [SDK Installation](https://github.com/cloudinary/analysis-python/blob/master/#sdk-installation)
  * [IDE Support](https://github.com/cloudinary/analysis-python/blob/master/#ide-support)
  * [SDK Example Usage](https://github.com/cloudinary/analysis-python/blob/master/#sdk-example-usage)
  * [Authentication](https://github.com/cloudinary/analysis-python/blob/master/#authentication)
  * [Available Resources and Operations](https://github.com/cloudinary/analysis-python/blob/master/#available-resources-and-operations)
  * [Retries](https://github.com/cloudinary/analysis-python/blob/master/#retries)
  * [Error Handling](https://github.com/cloudinary/analysis-python/blob/master/#error-handling)
  * [Server Selection](https://github.com/cloudinary/analysis-python/blob/master/#server-selection)
  * [Custom HTTP Client](https://github.com/cloudinary/analysis-python/blob/master/#custom-http-client)
  * [Resource Management](https://github.com/cloudinary/analysis-python/blob/master/#resource-management)
  * [Debugging](https://github.com/cloudinary/analysis-python/blob/master/#debugging)
* [Development](https://github.com/cloudinary/analysis-python/blob/master/#development)
  * [Maturity](https://github.com/cloudinary/analysis-python/blob/master/#maturity)
  * [Contributions](https://github.com/cloudinary/analysis-python/blob/master/#contributions)

<!-- End Table of Contents [toc] -->

<!-- Start SDK Installation [installation] -->
## SDK Installation

> [!NOTE]
> **Python version upgrade policy**
>
> Once a Python version reaches its [official end of life date](https://devguide.python.org/versions/), a 3-month grace period is provided for users to upgrade. Following this grace period, the minimum python version supported in the SDK will be updated.

The SDK can be installed with either *pip* or *poetry* package managers.

### PIP

*PIP* is the default package installer for Python, enabling easy installation and management of packages from PyPI via the command line.

```bash
pip install cloudinary-analysis
```

### Poetry

*Poetry* is a modern tool that simplifies dependency management and package publishing by using a single `pyproject.toml` file to handle project metadata and dependencies.

```bash
poetry add cloudinary-analysis
```
<!-- End SDK Installation [installation] -->

<!-- Start IDE Support [idesupport] -->
## IDE Support

### PyCharm

Generally, the SDK will work well with most IDEs out of the box. However, when using PyCharm, you can enjoy much better integration with Pydantic by installing an additional plugin.

- [PyCharm Pydantic Plugin](https://docs.pydantic.dev/latest/integrations/pycharm/)
<!-- End IDE Support [idesupport] -->

<!-- Start SDK Example Usage [usage] -->
## SDK Example Usage

### Example

```python
# Synchronous Example
import cloudinary_analysis
from cloudinary_analysis import CloudinaryAnalysis

with CloudinaryAnalysis(
    security=cloudinary_analysis.Security(
        cloudinary_auth=cloudinary_analysis.SchemeCloudinaryAuth(
            api_key="CLOUDINARY_API_KEY",
            api_secret="CLOUDINARY_API_SECRET",
        ),
    ),
) as cloudinary_analysis:

    res = cloudinary_analysis.analyze.ai_vision_general(source={
        "uri": "https://res.cloudinary.com/demo/image/upload/sample.jpg",
    }, notification_url="https://path.to/webhook", prompts=[
        "Describe this image in detail",
        "Does this image contain an insect?",
    ])

    # Handle response
    print(res)
```

</br>

The same SDK client can also be used to make asychronous requests by importing asyncio.
```python
# Asynchronous Example
import asyncio
import cloudinary_analysis
from cloudinary_analysis import CloudinaryAnalysis

async def main():
    async with CloudinaryAnalysis(
        security=cloudinary_analysis.Security(
            cloudinary_auth=cloudinary_analysis.SchemeCloudinaryAuth(
                api_key="CLOUDINARY_API_KEY",
                api_secret="CLOUDINARY_API_SECRET",
            ),
        ),
    ) as cloudinary_analysis:

        res = await cloudinary_analysis.analyze.ai_vision_general_async(source={
            "uri": "https://res.cloudinary.com/demo/image/upload/sample.jpg",
        }, notification_url="https://path.to/webhook", prompts=[
            "Describe this image in detail",
            "Does this image contain an insect?",
        ])

        # Handle response
        print(res)

asyncio.run(main())
```
<!-- End SDK Example Usage [usage] -->

<!-- Start Authentication [security] -->
## Authentication

### Per-Client Security Schemes

This SDK supports the following security schemes globally:

| Name              | Type   | Scheme       | Environment Variable         |
| ----------------- | ------ | ------------ | ---------------------------- |
| `cloudinary_auth` | http   | Custom HTTP  | `CLOUDINARY_CLOUDINARY_AUTH` |
| `o_auth2`         | oauth2 | OAuth2 token | `CLOUDINARY_O_AUTH2`         |

You can set the security parameters through the `security` optional parameter when initializing the SDK client instance. The selected scheme will be used by default to authenticate with the API for all operations that support it. For example:
```python
import cloudinary_analysis
from cloudinary_analysis import CloudinaryAnalysis

with CloudinaryAnalysis(
    security=cloudinary_analysis.Security(
        cloudinary_auth=cloudinary_analysis.SchemeCloudinaryAuth(
            api_key="CLOUDINARY_API_KEY",
            api_secret="CLOUDINARY_API_SECRET",
        ),
    ),
) as cloudinary_analysis:

    res = cloudinary_analysis.analyze.ai_vision_general(source={
        "uri": "https://res.cloudinary.com/demo/image/upload/sample.jpg",
    }, notification_url="https://path.to/webhook", prompts=[
        "Describe this image in detail",
        "Does this image contain an insect?",
    ])

    # Handle response
    print(res)

```
<!-- End Authentication [security] -->

<!-- Start Available Resources and Operations [operations] -->
## Available Resources and Operations

<details open>
<summary>Available methods</summary>

### [analyze](https://github.com/cloudinary/analysis-python/blob/master/docs/sdks/analyze/README.md)

* [ai_vision_general](https://github.com/cloudinary/analysis-python/blob/master/docs/sdks/analyze/README.md#ai_vision_general) - Analyze - AI Vision General
* [ai_vision_moderation](https://github.com/cloudinary/analysis-python/blob/master/docs/sdks/analyze/README.md#ai_vision_moderation) - Analyze - AI Vision Moderation
* [ai_vision_tagging](https://github.com/cloudinary/analysis-python/blob/master/docs/sdks/analyze/README.md#ai_vision_tagging) - Analyze - AI Vision Tagging
* [captioning](https://github.com/cloudinary/analysis-python/blob/master/docs/sdks/analyze/README.md#captioning) - Analyze - Captioning
* [cld_fashion](https://github.com/cloudinary/analysis-python/blob/master/docs/sdks/analyze/README.md#cld_fashion) - Analyze - Cld-Fashion
* [cld_text](https://github.com/cloudinary/analysis-python/blob/master/docs/sdks/analyze/README.md#cld_text) - Analyze - Cld-Text
* [coco](https://github.com/cloudinary/analysis-python/blob/master/docs/sdks/analyze/README.md#coco) - Analyze - Coco
* [google_logo_detection](https://github.com/cloudinary/analysis-python/blob/master/docs/sdks/analyze/README.md#google_logo_detection) - Analyze - Google Logo Detection
* [google_tagging](https://github.com/cloudinary/analysis-python/blob/master/docs/sdks/analyze/README.md#google_tagging) - Analyze - Google Tagging
* [human_anatomy](https://github.com/cloudinary/analysis-python/blob/master/docs/sdks/analyze/README.md#human_anatomy) - Analyze - Human Anatomy
* [image_quality](https://github.com/cloudinary/analysis-python/blob/master/docs/sdks/analyze/README.md#image_quality) - Analyze - Image Quality Analysis
* [lvis](https://github.com/cloudinary/analysis-python/blob/master/docs/sdks/analyze/README.md#lvis) - Analyze - Lvis
* [shop_classifier](https://github.com/cloudinary/analysis-python/blob/master/docs/sdks/analyze/README.md#shop_classifier) - Analyze - Shop Classifier
* [unidet](https://github.com/cloudinary/analysis-python/blob/master/docs/sdks/analyze/README.md#unidet) - Analyze - Unidet
* [watermark_detection](https://github.com/cloudinary/analysis-python/blob/master/docs/sdks/analyze/README.md#watermark_detection) - Analyze - Watermark Detection


### [tasks](https://github.com/cloudinary/analysis-python/blob/master/docs/sdks/tasks/README.md)

* [get_status](https://github.com/cloudinary/analysis-python/blob/master/docs/sdks/tasks/README.md#get_status) - Get analysis task status

</details>
<!-- End Available Resources and Operations [operations] -->

<!-- Start Retries [retries] -->
## Retries

Some of the endpoints in this SDK support retries. If you use the SDK without any configuration, it will fall back to the default retry strategy provided by the API. However, the default retry strategy can be overridden on a per-operation basis, or across the entire SDK.

To change the default retry strategy for a single API call, simply provide a `RetryConfig` object to the call:
```python
import cloudinary_analysis
from cloudinary_analysis import CloudinaryAnalysis
from cloudinary_analysis.utils import BackoffStrategy, RetryConfig

with CloudinaryAnalysis(
    security=cloudinary_analysis.Security(
        cloudinary_auth=cloudinary_analysis.SchemeCloudinaryAuth(
            api_key="CLOUDINARY_API_KEY",
            api_secret="CLOUDINARY_API_SECRET",
        ),
    ),
) as cloudinary_analysis:

    res = cloudinary_analysis.analyze.ai_vision_general(source={
        "uri": "https://res.cloudinary.com/demo/image/upload/sample.jpg",
    }, notification_url="https://path.to/webhook", prompts=[
        "Describe this image in detail",
        "Does this image contain an insect?",
    ],
        RetryConfig("backoff", BackoffStrategy(1, 50, 1.1, 100), False))

    # Handle response
    print(res)

```

If you'd like to override the default retry strategy for all operations that support retries, you can use the `retry_config` optional parameter when initializing the SDK:
```python
import cloudinary_analysis
from cloudinary_analysis import CloudinaryAnalysis
from cloudinary_analysis.utils import BackoffStrategy, RetryConfig

with CloudinaryAnalysis(
    retry_config=RetryConfig("backoff", BackoffStrategy(1, 50, 1.1, 100), False),
    security=cloudinary_analysis.Security(
        cloudinary_auth=cloudinary_analysis.SchemeCloudinaryAuth(
            api_key="CLOUDINARY_API_KEY",
            api_secret="CLOUDINARY_API_SECRET",
        ),
    ),
) as cloudinary_analysis:

    res = cloudinary_analysis.analyze.ai_vision_general(source={
        "uri": "https://res.cloudinary.com/demo/image/upload/sample.jpg",
    }, notification_url="https://path.to/webhook", prompts=[
        "Describe this image in detail",
        "Does this image contain an insect?",
    ])

    # Handle response
    print(res)

```
<!-- End Retries [retries] -->

<!-- Start Error Handling [errors] -->
## Error Handling

Handling errors in this SDK should largely match your expectations. All operations return a response object or raise an exception.

By default, an API error will raise a models.APIError exception, which has the following properties:

| Property        | Type             | Description           |
|-----------------|------------------|-----------------------|
| `.status_code`  | *int*            | The HTTP status code  |
| `.message`      | *str*            | The error message     |
| `.raw_response` | *httpx.Response* | The raw HTTP response |
| `.body`         | *str*            | The response content  |

When custom error responses are specified for an operation, the SDK may also raise their associated exceptions. You can refer to respective *Errors* tables in SDK docs for more details on possible exception types for each operation. For example, the `ai_vision_general_async` method may raise the following exceptions:

| Error Type                 | Status Code        | Content Type     |
| -------------------------- | ------------------ | ---------------- |
| models.ErrorResponse       | 400, 401, 403, 404 | application/json |
| models.RateLimitedResponse | 429                | application/json |
| models.ErrorResponse       | 500                | application/json |
| models.APIError            | 4XX, 5XX           | \*/\*            |

### Example

```python
import cloudinary_analysis
from cloudinary_analysis import CloudinaryAnalysis, models

with CloudinaryAnalysis(
    security=cloudinary_analysis.Security(
        cloudinary_auth=cloudinary_analysis.SchemeCloudinaryAuth(
            api_key="CLOUDINARY_API_KEY",
            api_secret="CLOUDINARY_API_SECRET",
        ),
    ),
) as cloudinary_analysis:
    res = None
    try:

        res = cloudinary_analysis.analyze.ai_vision_general(source={
            "uri": "https://res.cloudinary.com/demo/image/upload/sample.jpg",
        }, notification_url="https://path.to/webhook", prompts=[
            "Describe this image in detail",
            "Does this image contain an insect?",
        ])

        # Handle response
        print(res)

    except models.ErrorResponse as e:
        # handle e.data: models.ErrorResponseData
        raise(e)
    except models.RateLimitedResponse as e:
        # handle e.data: models.RateLimitedResponseData
        raise(e)
    except models.ErrorResponse as e:
        # handle e.data: models.ErrorResponseData
        raise(e)
    except models.APIError as e:
        # handle exception
        raise(e)
```
<!-- End Error Handling [errors] -->

<!-- Start Server Selection [server] -->
## Server Selection

### Server Variables

The default server `https://api.cloudinary.com/v2/analysis/{cloud_name}` contains variables and is set to `https://api.cloudinary.com/v2/analysis/CLOUD_NAME` by default. To override default values, the following parameters are available when initializing the SDK client instance:
 * `cloud_name: str`

### Override Server URL Per-Client

The default server can also be overridden globally by passing a URL to the `server_url: str` optional parameter when initializing the SDK client instance. For example:
```python
import cloudinary_analysis
from cloudinary_analysis import CloudinaryAnalysis

with CloudinaryAnalysis(
    server_url="https://api.cloudinary.com/v2/analysis/CLOUD_NAME",
    security=cloudinary_analysis.Security(
        cloudinary_auth=cloudinary_analysis.SchemeCloudinaryAuth(
            api_key="CLOUDINARY_API_KEY",
            api_secret="CLOUDINARY_API_SECRET",
        ),
    ),
) as cloudinary_analysis:

    res = cloudinary_analysis.analyze.ai_vision_general(source={
        "uri": "https://res.cloudinary.com/demo/image/upload/sample.jpg",
    }, notification_url="https://path.to/webhook", prompts=[
        "Describe this image in detail",
        "Does this image contain an insect?",
    ])

    # Handle response
    print(res)

```
<!-- End Server Selection [server] -->

<!-- Start Custom HTTP Client [http-client] -->
## Custom HTTP Client

The Python SDK makes API calls using the [httpx](https://www.python-httpx.org/) HTTP library.  In order to provide a convenient way to configure timeouts, cookies, proxies, custom headers, and other low-level configuration, you can initialize the SDK client with your own HTTP client instance.
Depending on whether you are using the sync or async version of the SDK, you can pass an instance of `HttpClient` or `AsyncHttpClient` respectively, which are Protocol's ensuring that the client has the necessary methods to make API calls.
This allows you to wrap the client with your own custom logic, such as adding custom headers, logging, or error handling, or you can just pass an instance of `httpx.Client` or `httpx.AsyncClient` directly.

For example, you could specify a header for every request that this sdk makes as follows:
```python
from cloudinary_analysis import CloudinaryAnalysis
import httpx

http_client = httpx.Client(headers={"x-custom-header": "someValue"})
s = CloudinaryAnalysis(client=http_client)
```

or you could wrap the client with your own custom logic:
```python
from cloudinary_analysis import CloudinaryAnalysis
from cloudinary_analysis.httpclient import AsyncHttpClient
import httpx

class CustomClient(AsyncHttpClient):
    client: AsyncHttpClient

    def __init__(self, client: AsyncHttpClient):
        self.client = client

    async def send(
        self,
        request: httpx.Request,
        *,
        stream: bool = False,
        auth: Union[
            httpx._types.AuthTypes, httpx._client.UseClientDefault, None
        ] = httpx.USE_CLIENT_DEFAULT,
        follow_redirects: Union[
            bool, httpx._client.UseClientDefault
        ] = httpx.USE_CLIENT_DEFAULT,
    ) -> httpx.Response:
        request.headers["Client-Level-Header"] = "added by client"

        return await self.client.send(
            request, stream=stream, auth=auth, follow_redirects=follow_redirects
        )

    def build_request(
        self,
        method: str,
        url: httpx._types.URLTypes,
        *,
        content: Optional[httpx._types.RequestContent] = None,
        data: Optional[httpx._types.RequestData] = None,
        files: Optional[httpx._types.RequestFiles] = None,
        json: Optional[Any] = None,
        params: Optional[httpx._types.QueryParamTypes] = None,
        headers: Optional[httpx._types.HeaderTypes] = None,
        cookies: Optional[httpx._types.CookieTypes] = None,
        timeout: Union[
            httpx._types.TimeoutTypes, httpx._client.UseClientDefault
        ] = httpx.USE_CLIENT_DEFAULT,
        extensions: Optional[httpx._types.RequestExtensions] = None,
    ) -> httpx.Request:
        return self.client.build_request(
            method,
            url,
            content=content,
            data=data,
            files=files,
            json=json,
            params=params,
            headers=headers,
            cookies=cookies,
            timeout=timeout,
            extensions=extensions,
        )

s = CloudinaryAnalysis(async_client=CustomClient(httpx.AsyncClient()))
```
<!-- End Custom HTTP Client [http-client] -->

<!-- Start Resource Management [resource-management] -->
## Resource Management

The `CloudinaryAnalysis` class implements the context manager protocol and registers a finalizer function to close the underlying sync and async HTTPX clients it uses under the hood. This will close HTTP connections, release memory and free up other resources held by the SDK. In short-lived Python programs and notebooks that make a few SDK method calls, resource management may not be a concern. However, in longer-lived programs, it is beneficial to create a single SDK instance via a [context manager][context-manager] and reuse it across the application.

[context-manager]: https://docs.python.org/3/reference/datamodel.html#context-managers

```python
import cloudinary_analysis
from cloudinary_analysis import CloudinaryAnalysis
def main():
    with CloudinaryAnalysis(
        security=cloudinary_analysis.Security(
            cloudinary_auth=cloudinary_analysis.SchemeCloudinaryAuth(
                api_key="CLOUDINARY_API_KEY",
                api_secret="CLOUDINARY_API_SECRET",
            ),
        ),
    ) as cloudinary_analysis:
        # Rest of application here...


# Or when using async:
async def amain():
    async with CloudinaryAnalysis(
        security=cloudinary_analysis.Security(
            cloudinary_auth=cloudinary_analysis.SchemeCloudinaryAuth(
                api_key="CLOUDINARY_API_KEY",
                api_secret="CLOUDINARY_API_SECRET",
            ),
        ),
    ) as cloudinary_analysis:
        # Rest of application here...
```
<!-- End Resource Management [resource-management] -->

<!-- Start Debugging [debug] -->
## Debugging

You can setup your SDK to emit debug logs for SDK requests and responses.

You can pass your own logger class directly into your SDK.
```python
from cloudinary_analysis import CloudinaryAnalysis
import logging

logging.basicConfig(level=logging.DEBUG)
s = CloudinaryAnalysis(debug_logger=logging.getLogger("cloudinary_analysis"))
```

You can also enable a default debug logger by setting an environment variable `CLOUDINARY_DEBUG` to true.
<!-- End Debugging [debug] -->

<!-- Placeholder for Future Speakeasy SDK Sections -->

# Development

## Maturity

This SDK is in beta, and there may be breaking changes between versions without a major version update. Therefore, we recommend pinning usage
to a specific package version. This way, you can install the same version each time without breaking changes unless you are intentionally
looking for the latest version.

## Contributions

While we value open-source contributions to this SDK, this library is generated programmatically. Any manual changes added to internal files will be overwritten on the next generation. 
We look forward to hearing your feedback. Feel free to open a PR or an issue with a proof of concept and we'll do our best to include it in a future release. 

### SDK Created by [Speakeasy](https://www.speakeasy.com/?utm_source=cloudinary-analysis&utm_campaign=python)
